/*
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o File        o I2c.h                                           
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
o Author      o EM Microelectronic-Marin SA
o Company     o EM Microelectronic-Marin SA                                                                            
o Project     o skeleton EM6812                                      
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo                                                                                                                                                                      
o Comments    o Some I2cs usefull definition - dependencie file
o             o
o             o                                     
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o Version     o 1.0
o Date        o 10.03.06
o Change      o Initial                                          
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
*/

// Definitions section
#ifndef __I2c__
#define __I2c__

// Include section
#include "Types.h"

/*** usefull I2C definitions ***/
// Define I2C bus on a standard Port IO - Bit 6/7
#define SdaI2C                    BIT7
#define SclI2C                    BIT6
#define RegOpenDrainI2C           RegOpenDrainPB
#define RegIOSelI2C               RegIOSelPB
#define RegPullDownI2C            RegPullDownPB
#define RegPullUpI2C              RegPullUpPB
#define RegOutI2C                 RegOutPB
#define RegInI2C                  RegInPB
#define CheckComingAckI2C         if ((RegInI2C & SdaI2C) == SdaI2C) { (VarStatusRegI2C |= ErrorStatusI2C); (VarStatusRegI2C &= ~AckStatusI2C); return;}
#define SetAckStatusI2C           VarStatusRegI2C |= AckStatusI2C
#define ClrAckStatusI2C           VarStatusRegI2C &= ~AckStatusI2C
#define SetRxModeStatusI2C        VarStatusRegI2C |= RxModeStatusI2C
#define ClrRxModeStatusI2C        VarStatusRegI2C &= ~RxModeStatusI2C
#define CheckAckStatusI2C         ((VarStatusRegI2C & AckStatusI2C) == AckStatusI2C)
#define CheckErrorStatusI2C       ((VarStatusRegI2C & ErrorStatusI2C) == ErrorStatusI2C) 
#define CheckRxModeStatusI2C      ((VarStatusRegI2C & RxModeStatusI2C) == RxModeStatusI2C)
#define ErrorStatusI2C            BIT7
#define RxModeStatusI2C           BIT1
#define AckStatusI2C              BIT0                     
#define MsbI2C                    BIT7
#define TestMsbTxI2C              ((VarShiftRegI2C & MsbI2C) == MsbI2C)
#define ShiftLeftVarShiftRegI2C   (VarShiftRegI2C = (VarShiftRegI2C << 1))
#define ReadSlaveI2C              1
#define TestSclI2CHigh            ((RegInI2C & SclI2C) == SclI2C)
#define TestSdaI2CHigh            ((RegInI2C & SdaI2C) == SdaI2C)
#define LoadOneVarShiftRegI2C     VarShiftRegI2C |= 1
#define LoadZeroVarShiftRegI2C    VarShiftRegI2C |= 0
#define ReleaseI2C                RegIOSelI2C &= ~(SdaI2C | SclI2C)   // both pins not driven
#define ForceZeroI2C              RegIOSelI2C |= (SdaI2C | SclI2C)   // both pins to '0'  
#define ReleaseSclI2C             RegIOSelI2C &= ~SclI2C // SclI2C not driven
#define ForceZeroSclI2C           RegIOSelI2C |= SclI2C  // SclI2C to '0' 
#define ReleaseSdaI2C             RegIOSelI2C &= ~SdaI2C // SdaI2C not driven   
#define ForceZeroSdaI2C           RegIOSelI2C |= SdaI2C  // SdaI2C to '0' 
#define DeviceMemoryI2C           YES
#define CheckDeviceMemoryI2C      (DeviceTypeI2C & DeviceMemoryI2C) == DeviceMemoryI2C                            
#define YES				                1
#define NO				                0
#define ResultOperationKO      ((ResultOperation & ErrorStatusI2C) == ErrorStatusI2C)

/*** I2C functions ***/
void WaitTimeI2C(UINT8 delay_size);
void InitPortI2C(void);
void StartConditionI2C(void);
void StopConditionI2C(void);
void CheckSlaveAcknowledgeI2C(void);
void NoMasterAcknowledgeI2C(void);
void MasterAcknowledgeI2C(void);
void SendBitI2C(void);
void GetBitI2C(void);
unsigned char SendBufferI2C(unsigned char* pBufferOut, unsigned char BufferDepth, unsigned char DeviceTypeI2C);             
unsigned char GetBufferI2C(unsigned char* pBufferIn, unsigned char BufferDepth);             
       

       
       
       
                  
#endif /* __I2c__ */

