/*
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o File        o crt0.ld                                           
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
o Author      o EM Microelectronic-Marin SA
o Company     o EM Microelectronic-Marin SA                                                                            
o Project     o skeleton EM6812                                      
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo                                                                                                                                                                      
o Comments    o CoolRisc 816 linker script
o             o
o             o                                     
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
o Version     o 1.0
o Date        o 10.11.04
o Change      o Initial                                          
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 

oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
o Linker script functions
o
o Initialize the different memory areas
o Give address to each peripheral register
o Relocate stack pointer
o Place each compiler section into a certain memory area 
o
o Memory area definition 
o 
o prog_rom : program memory space (8k max = 0x7fff)
o ram_lp   : internal data low-power RAM page 0 space
o ram_dp   : internal data dual port RAM page 0 space 
o periph_1 : internal peripheral addresses page 0 space
o periph_2 : internal peripheral addresses page 0 space
o periph_3 : internal peripheral addresses page 0 space
o periph_4 : internal peripheral addresses page 0 space
o ram_1    : internal data RAM page 0 space
o ram_2    : internal data RAM space
o
o Sections definition: 
o 
o .text         : program space, ELF's 0x100000..0x107fff addressing range
o .rodata       : read-only data
o .data         : data with initialization value
o .bss          : data without initialization value
o .page0_data   : page 0 data with initialization value
o .page0_bss    : page 0 data without initialization value
o
oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo 
*/

MEMORY
{
  prog_rom          : ORIGIN = 0x00100000, LENGTH = 4*8K   
  ram_lp            : ORIGIN = 0x00000000, LENGTH = 12
  ram_dp            : ORIGIN = 0x0000000C, LENGTH = 4
  periph_1          : ORIGIN = 0x00000010, LENGTH = 14         
  periph_2          : ORIGIN = 0x00000020, LENGTH = 12
  periph_3          : ORIGIN = 0x00000030, LENGTH = 13
  periph_4          : ORIGIN = 0x0000004E, LENGTH = 18
  ram_1             : ORIGIN = 0x00000060, LENGTH = 0xA0 /* 160 */
  ram_2             : ORIGIN = 0x00000100, LENGTH = 0x15F /* 352*/
  error_nodata_rom  : ORIGIN = 0x00001000, LENGTH = 0
 }
 
/*
 DATA ROM :
The current chip has no data rom (= initialized data loaded in
ROM, EEPROM or Flash memory area). DataInit mecanism allow to
transfert initialized data into the RAM area.
For this reason, the .*data sections are not defined as null  
*/
 
 /*
--------------+------------------------------------------------------
              |0x00
              |...				ram_lp + ram_dp + Periphery_1 to 4 
              |0x5F
Page0(Direct)-------------------------------------------------------
              |0x60
              |...    ram_1																														
              |0xFF
--------------+------------------------------------------------------
 													|0x100
Page1(Indexed)|...			ram_2																													
 													|0x1FF
--------------+------------------------------------------------------
														|0x200
														|...			ram_2																														
														|0x25F
Page2(Indexed)-------------------------------------------------------
														|0x260
														|...			Not exist
														|0x2FF
--------------+------------------------------------------------------
*/ 
 

/* 
Peripherical register address definition
*/

/* 
ram_lp and ram_dp
In this default ctrl0.ld file, ram_lp and ram_dp are let undefined. They
can be defined as registers or ram section. It is let to the user's 
choice. 
By default they are let defined as user registers ... 
*/
LPRam0   = 0x00;
LPRam1   = 0x01;
LPRam2   = 0x02;
LPRam3   = 0x03;
LPRam4   = 0x04;
LPRam5   = 0x05;
LPRam6   = 0x06;
LPRam7   = 0x07;
LPRam8   = 0x08;
LPRam9   = 0x09;
LPRam10  = 0x0A;
LPRam11  = 0x0B;   
DPRam0   = 0x0C;
DPRam1   = 0x0D;
DPRam2   = 0x0E;
DPRam3   = 0x0F; 

/*
System registers
*/
RegSys1           = 0x10; 
RegSys2           = 0x11;
RegResStat        = 0x12;
RegTrimRC         = 0x13;
RegPrCkSel      = 0x14;
RegPr1Status      = 0x15;
RegPr2Status      = 0x16;
RegInt20          = 0x17;
RegInt10          = 0x18;
RegInt00          = 0x19;
RegMsk20          = 0x1A;
RegMsk10          = 0x1B;
RegMsk00          = 0x1C;

/*
Analog
*/
RegAnaCfg         = 0x20;

/*
Port A
*/
RegInPA           = 0x21;
RegOutPA          = 0x22;
RegCfgPA          = 0x23;
RegIOSelPA        = 0x24;
RegCmbKey         = 0x25;
RegMskRstWkUp     = 0x26;
RegIntEdgPAPA     = 0x27;
RegEnDebPA        = 0x28;
RegOpenDrainPA    = 0x29;
RegPullUpPA       = 0x2A;
RegPullDownPA     = 0x2B;
    
/* 
Port B
*/
RegInPB           = 0x30;
RegOutPB          = 0x31;
RegCfgPB          = 0x32;
RegSigSel         = 0x33;
RegIOSelPB        = 0x34;
RegOpenDrainPB    = 0x35;
RegPullUpPB       = 0x36;
RegPullDownPB     = 0x37;

/*
SPI
*/      
RegSPICfg         = 0x38;
RegSPIDat         = 0x39;
RegSPILoad        = 0x3A;

/*
WD
*/
RegWDSys          = 0x3B;
RegWDKey          = 0x3C;

/*
General Timer
*/
RegTimersCfg      = 0x4E;
RegTimersStart    = 0x4F;

/*
Timer1
*/
RegTim1Cfg        = 0x50;
RegTim1Status     = 0x51;
RegTim1Load       = 0x52;
RegTim1Comp       = 0x53;

/*
Timer2
*/
RegTim2Cfg        = 0x54;
RegTim2Status     = 0x55;
RegTim2Load       = 0x56;
RegTim2Comp       = 0x57;

/*
Timer3
*/
RegTim3Cfg        = 0x58;
RegTim3Status     = 0x59;
RegTim3Load       = 0x5A;
RegTim3Comp       = 0x5B;

/*
Timer4
*/
RegTim4Cfg        = 0x5C;
RegTim4Status     = 0x5D;
RegTim4Load       = 0x5E;
RegTim4Comp       = 0x5F;

         
/************************************************************************/
/* Support for old names definition
/************************************************************************/
RegRam0		      =	0x0C;
RegRam1		      =	0x0D;
RegRam2		      =	0x0E;
RegRam3		      =	0x0F; 
/********* System registers *********/
RegSys1	   	    = 0x10;	
RegSys2  	      =	0x11;
RegResStat      =	0x12;
RegTrimRC       =	0x13;
RegPrCkSel   	=	0x14;
RegPresc1Status	=	0x15;
RegPresc2Status	=	0x16;
RegInt20   	    =	0x17;
RegInt10   	    =	0x18;
RegInt00   	    =	0x19;
RegMsk20   	    =	0x1A;
RegMsk10   	    =	0x1B;
RegMsk00   	    =	0x1C;
RegGB		        =	0x1D;	
/********* Analog *********/
RegAnaCfg      	=	0x20;
/********* Port A *********/
RegInPA		      =	0x21;
RegOutPA   	    =	0x22;
RegCfgPA 	      =	0x23;
RegIOSelPA    	=	0x24;
RegCmbPA    	  =	0x25;
RegMskPA	      =	0x26;
RegIntEdgPA   	=	0x27;
RegEnDebPA  	  =	0x28;
RegOpenDrainPA	=	0x29;
RegPullUpPA   	=	0x2A;
RegPullDownPA	  =	0x2B;		
/********* Port B *********/
RegInPB   	    =	0x30;
RegOutPB  	    =	0x31;
RegCfgPB  	    =	0x32;
RegIntSel 	    =	0x33;
RegIOSelPB	    =	0x34;
RegOpenDrainPB	=	0x35;
RegPullUpPB   	=	0x36;
RegPullDownPB 	=	0x37;
/********* SPI *********/			
RegSPICfg 	    =	0x38;
RegSPIDat 	    =	0x39;
RegSPILoad	    =	0x3A;
/********* WD *********/
RegWDSys  	    =	0x3B;
RegWDKey  	    =	0x3C;
/********* General Timer *********/
RegTimersCfg	  = 0x4E;
RegTimersStart	= 0x4F;
/********* Timer1 *********/
RegTim1Cfg      =	0x50;
RegTim1Status   =	0x51;
RegTim1Load     =	0x52;
RegTim1Comp     =	0x53;
/********* Timer2 *********/
RegTim2Cfg 	    =	0x54;
RegTim2Status	  =	0x55;
RegTim2Load	    =	0x56;
RegTim2Comp     =	0x57;
/********* Timer3 *********/
RegTim3Cfg 	    =	0x58;
RegTim3Status	  =	0x59;
RegTim3Load	    =	0x5A;
RegTim3Comp     =	0x5B;
/********* Timer4 *********/
RegTim4Cfg 	    =	0x5C;
RegTim4Status	  =	0x5D;
RegTim4Load   	=	0x5E;
RegTim4Comp     =	0x5F;
/************************************************************************



/***********************************************************************
** Definition of the top of the stack
***********************************************************************/
/* stack is on RAM_2 */
_estack = 0x025F ;   

/***********************************************************************
** Place compiler sections into the various memory area
***********************************************************************/ 
SECTIONS 
{
    /* THE PROGRAM CODE IS STORED IN THE .TEXT SECTION */
    .text : 
    {
        _stext = .;
        *(.text )
        _etext = .;
        _eprom = .;
    } > prog_rom
    
    /* THE PAGE 0 UNINITIALIZED DATA ARE STORED INTO THE .PAGE0_BSS SECTION */
    .page0_bss :  
    {
        _spage0bss = .;
        *(.page0_bss)
        _epage0bss = .;
    } > ram_1

    /* THE UNINITIALIZED DATA ARE STORED INTO THE .BSS SECTION */
    .bss : 
    {
        _sbss = .;
        *(.bss)
        *(COMMON)
        _ebss = .;
    } > ram_2
    
    /* EMPTY SECTIONS BECAUSE NO DATA ROM BUT MUST BE DEFINED !!! */
    
    .rodata : /* AT ( ADDR(.bss)+ SIZEOF(.bss)) */
    {
        _srodata = .;
        *(.rodata)
        _erodata = .;
    /* } > error_nodata_rom */
    } > ram_2

    .page0_data  :  /* AT (ADDR (.rodata) + SIZEOF(.rodata)) */
    {
    _spage0data = .;
    *(.page0_data)
    _epage0data = .;
   /* } > error_nodata_rom */
    } > ram_1
    
    .data  :  AT (ADDR (.rodata) + SIZEOF(.rodata)+ SIZEOF(.page0_data))
    {
    _sdata = .;
    *(.data)
    _edata = .;
    /* } > error_nodata_rom */
    } > ram_2

    /* THESE SECTIONS CONTAIN DEBUG INFORMATION */
    
    .stab : 
    {
        *(.stab)
    }   

    .stabstr : 
    {
        *(.stabstr)
    }       
}



